"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = debug;

var _serializeError = _interopRequireDefault(require("serialize-error"));

var _repl = _interopRequireDefault(require("@wdio/repl"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(source, true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(source).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function debug(commandTimeout = 5000) {
  const repl = new _repl.default();
  const {
    introMessage
  } = _repl.default;

  if (!process.env.WDIO_WORKER) {
    console.log(_repl.default.introMessage);
    const context = {
      browser: this,
      driver: this,
      $: this.$.bind(this),
      $$: this.$$.bind(this)
    };
    return repl.start(context);
  }

  process._debugProcess(process.pid);

  process.send({
    origin: 'debugger',
    name: 'start',
    params: {
      commandTimeout,
      introMessage
    }
  });

  let commandResolve = () => {};

  process.on('message', m => {
    if (m.origin !== 'debugger') {
      return;
    }

    if (m.name === 'stop') {
      process._debugEnd(process.pid);

      return commandResolve();
    }

    if (m.name === 'eval') {
      repl.eval(m.content.cmd, global, null, (e, result) => {
        if (e) {
          process.send({
            origin: 'debugger',
            name: 'result',
            params: _objectSpread({
              error: true
            }, (0, _serializeError.default)(e))
          });
        }

        if (typeof result === 'function') {
          result = `[Function: ${result.name}]`;
        }

        process.send({
          origin: 'debugger',
          name: 'result',
          params: {
            result
          }
        });
      });
    }
  });
  return new Promise(resolve => commandResolve = resolve);
}